summaryrefslogtreecommitdiffstats
path: root/fs/ceph/mdsmap.c
diff options
context:
space:
mode:
authorXiubo Li <xiubli@redhat.com>2019-11-26 07:24:22 -0500
committerIlya Dryomov <idryomov@gmail.com>2020-01-27 16:53:39 +0100
commit5d47648fe95412beffe2089d6d6484adb5ea0f96 (patch)
tree11ec28f717549c83a44b4c294ec582bb8ae02d21 /fs/ceph/mdsmap.c
parent4d7ace02ba5c6ef1f8eeb32a86fef7c528bd7f36 (diff)
downloadlinux-5d47648fe95412beffe2089d6d6484adb5ea0f96.tar.bz2
ceph: only choose one MDS who is in up:active state without laggy
Even the MDS is in up:active state, but it also maybe laggy. Here will skip the laggy MDSs. Signed-off-by: Xiubo Li <xiubli@redhat.com> Reviewed-by: Jeff Layton <jlayton@kernel.org> Signed-off-by: Ilya Dryomov <idryomov@gmail.com>
Diffstat (limited to 'fs/ceph/mdsmap.c')
-rw-r--r--fs/ceph/mdsmap.c30
1 files changed, 23 insertions, 7 deletions
diff --git a/fs/ceph/mdsmap.c b/fs/ceph/mdsmap.c
index 7a925e025c0a..a77e0ecb9a6b 100644
--- a/fs/ceph/mdsmap.c
+++ b/fs/ceph/mdsmap.c
@@ -13,22 +13,24 @@
#include "super.h"
+#define CEPH_MDS_IS_READY(i, ignore_laggy) \
+ (m->m_info[i].state > 0 && (ignore_laggy ? true : !m->m_info[i].laggy))
-/*
- * choose a random mds that is "up" (i.e. has a state > 0), or -1.
- */
-int ceph_mdsmap_get_random_mds(struct ceph_mdsmap *m)
+static int __mdsmap_get_random_mds(struct ceph_mdsmap *m, bool ignore_laggy)
{
int n = 0;
int i, j;
/* special case for one mds */
if (1 == m->m_num_mds && m->m_info[0].state > 0)
return 0;
/* count */
for (i = 0; i < m->m_num_mds; i++)
- if (m->m_info[i].state > 0)
+ if (CEPH_MDS_IS_READY(i, ignore_laggy))
n++;
if (n == 0)
return -1;
@@ -36,7 +38,7 @@ int ceph_mdsmap_get_random_mds(struct ceph_mdsmap *m)
/* pick */
n = prandom_u32() % n;
for (j = 0, i = 0; i < m->m_num_mds; i++) {
- if (m->m_info[i].state > 0)
+ if (CEPH_MDS_IS_READY(i, ignore_laggy))
j++;
if (j > n)
break;
@@ -45,6 +47,20 @@ int ceph_mdsmap_get_random_mds(struct ceph_mdsmap *m)
return i;
}
+/*
+ * choose a random mds that is "up" (i.e. has a state > 0), or -1.
+ */
+int ceph_mdsmap_get_random_mds(struct ceph_mdsmap *m)
+{
+ int mds;
+
+ mds = __mdsmap_get_random_mds(m, false);
+ if (mds == m->m_num_mds || mds == -1)
+ mds = __mdsmap_get_random_mds(m, true);
+
+ return mds == m->m_num_mds ? -1 : mds;
+}
+
#define __decode_and_drop_type(p, end, type, bad) \
do { \
if (*p + sizeof(type) > end) \
@@ -13,22 +13,24 @@
int n = 0;
int i, j;
- /* special case for one mds */
+ /*
+ * special case for one mds, no matter it is laggy or
+ * not we have no choice
+ */
if (1 == m->m_num_mds && m->m_info[0].state > 0)
return 0;